#! perl -w
BEGIN {
    print"Begin: Original block!\n";
    print"OperatingSystem: $^O\n";
   
   use Cava::Packager;
    
    if(Cava::Pack::Packaged()) {
        my $command = join(' ', @ARGV);
        if($command =~ / -MIPC::Run::Win32Pump -e 1 \d+ \d+ \d+ \d+ \d+ \d+ \d+ /) {
            while( my $arg = shift( @ARGV )) {
                last if($arg eq '1');
            }
            eval{ require IPC::Run::Win32Pump; };
            die $@ if $@;
            exit(0);
	    print"Begin: IPC::Run::Win32Pump loaded.\n";
        }
        if($command =~ / -MIPC::Run::Win32Pump -e 1/) {
            die 'Incorrect IPC::Run handling in executable';
        }
    }
}


#initial
package SLS;

use strict;
use warnings;
use lib '.';

use Wx qw(:everything);
use Wx::Event qw(:everything);
use Wx::XRC;
use IPC::Run qw( start pump finish timeout harness );
#eval ("use IPC::Run::Win32Pump;");

use Cava::Packager;
use base qw(Wx::App);

my $ionecal = 1;
my $dowork = 0;
my $sleep = 1;
my $timer = '50';
my $ipcdebug = 1; # Set 1-3 for higher debugging output of IPC::Run module
my ($app, $xr, $frame);
my ($bindir, $resourcedir, $xrcfile);
our ($in, $out, $err, $t, $h);
our %case;

#setup env and vars
print "Seting up environment ...\n";

if(Cava::Pack::Packaged()) {
	$bindir = Cava::Packager::GetBinPath();
	$resourcedir = Cava::Packager::GetResourcePath();
} else {
 	$bindir = '.';
 	$resourcedir = './res';
}

$xrcfile = $resourcedir .'/gui.xrc';

print"Resource dir: $resourcedir\n";
print"Bin dir: \n";
print"XRC file: $xrcfile\n";

$ENV{PATH} = $ENV{PATH} . "$bindir";
$ENV{ARGYLL_NOT_INTERACTIVE} = 1;

#$ENV{IPCRUNDEBUG}='gory';
#if ($^O eq 'MSWin32') { system("set"); } else { system("env");}
	
#---------------------- Main script --------------------------#

sub OnInit {	
	$app   = shift;
    
    #Load xrc file for ui
	$xr = Wx::XmlResource->new();
	$xr->InitAllHandlers();
	$xr->Load($xrcfile);
	$frame = $xr->LoadFrame(undef, 'ID_WXFRAME');
	
	my $XID = \&Wx::XmlResource::GetXRCID;		
	
	EVT_BUTTON($frame, &$XID('ID_BUTTON_START'), sub { 
		$dowork = 1;
		refreshUi();
		ioProcessing(); 
		});
	
	EVT_BUTTON($frame, &$XID('ID_BUTTON_STOP'), sub { $dowork = 0; } );
	
	$frame->Show(1);                        # Fenster zeichnen
	$app->SetTopWindow($frame);             # Fenster als oberstes bestimmen

1;	
}

#------------------------- Subroutines -------------------------#

sub ioProcessing {
		
	print"\n#-------------------- Start ioProcessing -------------------- #\n";

	my @cmd = ("$bindir/fakespotread.exe");
	
	print"IPC Command: @cmd\n";	
	$h = start( \@cmd, \$in, \$out, \$err, ( $t = timeout($timer) ), debug => $ipcdebug);

	%case = (
	"error"   => \&ioMessage,
	"reading" => \&ioReading,
	"calibration" => \&ioCalibration,
	"complete" => \&ioComplete,
	"measured" => \&ioReading,
	);
	
	while ($dowork == 1) {
		print"Error: $err\n";
		
		if ($ionecal == 0) {
			$h->pump until $out =~ /calibration|complete|error/mgc; 
			$case{$&}->();								
		} else {
			$h->pump until $out =~ /reading|error/mgc;
			$case{$&}->();
		}
		
		refreshUi();
		sleep $sleep;				
	}
	
	$h->kill_kill;
	finish $h ; 	
}

sub ioMessage {
	print "#Debug:" . $out . "\n";
	
	print"\n#Found what you where looking for: $case{$&}\n";
}

sub ioComplete {
	print "#Debug:" . $out . "n";
	print"\n#Instrument calibration done!\n";
	$ionecal = 1;
}	

sub ioCalibration 	{
	print "#Debug:" . $out . "\n";
	
	print"\n#Press ENTER to start instrument calibration...\n";

	my $input = <STDIN>;
	print"Input was: $input\n";
	$t->start($timer);
	$in .= $input;
	$out = '';
}

sub ioReading {	
	#print "#Debug reading:" . $out . "\n";
	$t->start($timer);
	$in .= "c\n";
	$out = '';
}	

sub refreshUi {
	$frame->Update();
	$frame->Refresh();
	$app->Yield();
}

sub _parhints{	
	require ExtUtils::CBuilder;
	require IPC::Run::Win32Pump;
}

package main;
SLS->new->MainLoop;

__END__
BEGIN {	
	print"Begin: Enter begin block.\n";
	
	eval("use Cava::Packager;");
	
	if(Cava::Pack::Packaged()) {
	
		if ($^O eq 'MSWin32') {
	
			my $command = join(' ', @ARGV);
			
			print"Begin command 1: $command\n";
			print"Beginn ARGV: @ARGV\n";
			
			if($command =~ / -MIPC::Run::Win32Pump -e 1 \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ \d+ /) {
				
				print"Begin command 2: $command\n";
				
				while( my $arg = shift( @ARGV )) {
					last if($arg eq '1');
				}
				eval{ require IPC::Run::Win32Pump; };
				die $@ if $@;
				exit(0);
			}
			if($command =~ / -MIPC::Run::Win32Pump -e 1/) {
				die 'Incorrect IPC::Run handling in executable';
			}
		print"Begin: Windows detected.\n";
		}    
	print"Begin: Packed application.\n";	
	}
print"Begin: Leave begin block.\n";	
}

sub ioReading {	
	print "#Debug:" . $out . "\n";
	
	print"\n#Press ENTER to take reading...\n";	
	my $input = <STDIN>;	
	print"Input was: $input\n";
	$t->start($timer);
	$in .= $input;
	$out = '';		#do not move - mandatory here!
}	
